package com.lelab.ui;

import android.view.View;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorManager;
import android.hardware.SensorEventListener;
import android.util.Log;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.RectF;

public class AccView  extends View {

    private Bitmap  mBitmap;
    private Paint   mPaint = new Paint();
    private Canvas  mCanvas = new Canvas();
    private Path    mPath = new Path();
    private RectF   mRect = new RectF();
    private float   mLastValues[] = new float[3*2];
    private float   mOrientationValues[] = new float[3];
    private float   mAccValues[] = new float[3];
    private int     mColors[] = new int[3*2];
    private float   mLastX;
    private float   mScale[] = new float[2];
    private float   mYOffset;
    private float   mMaxX;
    private float   mSpeed = 1.0f;
    private float   mWidth;
    private float   mHeight;

    /** jumper code **/
    private float txtLine;
    private float[] a_x;
	private float[] a_y;
	private float[] a_z;
	private int index;
	private int xJump;
	private int yJump;
	private int zJump;
    /** end jumper code **/
    
    public AccView(Context context) {
        super(context);
        //Colors for accelerometer's values graph
        mColors[0] = Color.argb(192, 255, 64, 64); //X red
        mColors[1] = Color.argb(192, 64, 128, 64); //Y green
        mColors[2] = Color.argb(192, 64, 64, 255); //Z blue
        //Colors for magnetic field's values graph
        mColors[3] = Color.argb(192, 64, 255, 255); //cyan
        mColors[4] = Color.argb(192, 128, 64, 128); //magenta
        mColors[5] = Color.argb(192, 255, 255, 64); //yellow

        mPaint.setFlags(Paint.ANTI_ALIAS_FLAG);
        mRect.set(-0.5f, -0.5f, 0.5f, 0.5f);
        mPath.arcTo(mRect, 0, 180);
        
        /** jumper code **/
        a_x = new float[5];
        a_y = new float[5];
        a_z = new float[5];
        index = 0;
                
        xJump = 0;
        yJump = 0;
        zJump = 0;
        /** end jumper code **/
        
    }
    
    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        Log.d("SCREEN" , "width: " + String.valueOf(w)+ " ; Height: "+String.valueOf(h));
        mBitmap = Bitmap.createBitmap(w, h, Bitmap.Config.RGB_565);
        mCanvas.setBitmap(mBitmap);
        mCanvas.drawColor(0x000000FF);//(0xFFFFFFFF);
        mYOffset = h * 0.5f;
        mScale[0] = - (h * 0.5f * (1.0f / (SensorManager.STANDARD_GRAVITY * 2)));
        mScale[1] = - (h * 0.5f * (1.0f / (SensorManager.MAGNETIC_FIELD_EARTH_MAX)));
        mWidth = w;
        mHeight = h;
        if (mWidth < mHeight) {
            mMaxX = w;
        } else {
            mMaxX = w-50;
        }
        mLastX = mMaxX;
        super.onSizeChanged(w, h, oldw, oldh);
    }

    @Override
    protected void onDraw(Canvas canvas) {
    	super.onDraw(canvas);
        synchronized (this) {
            if (mBitmap != null) {
                final Paint paint = mPaint;
                final Path path = mPath;
                //Print the lines to separate each sensor's graph
                if (mLastX >= mMaxX) {
                    mLastX = 0;
                    final Canvas cavas = mCanvas;
                    final float yoffset = mYOffset;
                    final float maxx = mMaxX;
                    final float oneG = SensorManager.STANDARD_GRAVITY * mScale[0];
                    paint.setColor(0xFFAAAAAA);  //Grey
                    cavas.drawColor(0xFFFFFFFF); //White
                    cavas.drawLine(0, yoffset,      maxx, yoffset,      paint);
                    cavas.drawLine(0, yoffset+oneG, maxx, yoffset+oneG, paint);
                    cavas.drawLine(0, yoffset-oneG, maxx, yoffset-oneG, paint);
                }
                canvas.drawBitmap(mBitmap, 0, 0, null);

                //Print the orientation values in a circle-type graph
                float[] values = mOrientationValues;
                final int outerColor = 0xFFC0C0C0;  //Orange
                final int innerColor = 0xFFff7010;  //Grey
                // values[0]:
                // Azimuth, angle between the magnetic north direction and the Y axis,
                // around the Z axis (0 to 359). 0=North, 90=East, 180=South, 270=West
                // values[1]:
                // Pitch, rotation around X axis (-180 to 180),
                // with positive values when the z-axis moves toward the y-axis.
                // values[2]:
                // Roll, rotation around Y axis (-90 to 90),
                // with positive values when the x-axis moves away from the z-axis.             
                if (mWidth < mHeight) {
                    float w0 = mWidth * 0.333333f;
                    float w  = w0 - 32;
                    float x = w0*0.5f;
                    for (int i=0 ; i<3 ; i++) {
                        canvas.save(Canvas.MATRIX_SAVE_FLAG);
                        canvas.translate(x, w*0.5f + 4.0f);
                        canvas.save(Canvas.MATRIX_SAVE_FLAG);
                        paint.setColor(outerColor);
                        canvas.scale(w, w);
                        canvas.drawOval(mRect, paint);
                        canvas.restore();
                        canvas.scale(w-5, w-5);
                        paint.setColor(innerColor);
                        canvas.rotate(-values[i]);
                        canvas.drawPath(path, paint);
                        canvas.restore();
                        x += w0;
                    }
                } else {
                    float h0 = mHeight * 0.333333f;
                    float h  = h0 - 32;
                    float y = h0*0.5f;
                    for (int i=0 ; i<3 ; i++) {
                        canvas.save(Canvas.MATRIX_SAVE_FLAG);
                        canvas.translate(mWidth - (h*0.5f + 4.0f), y);
                        canvas.save(Canvas.MATRIX_SAVE_FLAG);
                        paint.setColor(outerColor);
                        canvas.scale(h, h);
                        canvas.drawOval(mRect, paint);
                        canvas.restore();
                        canvas.scale(h-5, h-5);
                        paint.setColor(innerColor);
                        canvas.rotate(-values[i]);
                        canvas.drawPath(path, paint);
                        canvas.restore();
                        y += h0;
                    }
                }
            }
        }
    }

	public void onSensorChanged(SensorEvent event) {
		//Print on the Log window the values of sensor
//        Log.d("SENSOR" , "sensor: " + event.sensor.getName() + " ; " + event.sensor.getType() + " ; " + event.sensor.getResolution() + " [a.u.] ; " + event.sensor.getPower() + " mA" + ", x: " + String.valueOf(event.values[0]) + ", y: " + String.valueOf(event.values[1]) + ", z: " + String.valueOf(event.values[2]));
        
        //To synchronize the simultaneous collection of different sensors 
        synchronized (this) {
            if (mBitmap != null) {
                final Canvas canvas = mCanvas;
                final Paint paint = mPaint;                
                if (event.sensor.getType() == Sensor.TYPE_ORIENTATION) {
                    for (int i=0 ; i<3 ; i++) {
                        mOrientationValues[i] = event.values[i];
                    }
                }                
                else if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER){
                    float deltaX = mSpeed;
                    float newX = mLastX + deltaX;
                    for (int i=0 ; i<3 ; i++) {
                        final float v = mYOffset + event.values[i] * mScale[0];
                        mAccValues[i] = event.values[i];
                        paint.setColor(mColors[i]);
                        canvas.drawLine(mLastX, mLastValues[i], newX, v, paint);
                        mLastValues[i] = v;
                    }
                    mLastX += mSpeed;                                
                }
                //Force the view to draw
                invalidate();
            }
        }		
	}
	
	public void onRemoteSensorChanged(float[] values) {
		//Print on the Log window the values of sensor
        
        
        if (mBitmap != null) {
        	final Canvas canvas = mCanvas;
            final Paint paint = mPaint;                
            
            float deltaX = mSpeed;
            float newX = mLastX + deltaX;
            for (int i=0 ; i<3 ; i++) {
                final float v = mYOffset + values[i] * mScale[0];
                mAccValues[i] = values[i];
                paint.setColor(mColors[i]);
                canvas.drawLine(mLastX, mLastValues[i], newX, v, paint);
                mLastValues[i] = v;
            }
            mLastX += mSpeed;        
            //Force the view to draw
            invalidate();
        }
	}	

}
